<?php

/**
 * Copyright Souke (2018)
 *
 * contact@souke.fr
 *
 * Ce logiciel est un programme informatique servant à aider les producteurs
 * à distribuer leur production en circuits courts.
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 */

namespace backend\controllers;

use common\helpers\GlobalParam;
use domain\Distribution\PointSaleDistribution\PointSaleDistribution;
use domain\Order\Order\Order;
use domain\PointSale\PointSale\PointSale;
use domain\PointSale\PointSale\PointSaleSearch;
use domain\PointSale\UserPointSale\UserPointSale;
use yii\filters\AccessControl;
use yii\helpers\Html;
use yii\web\NotFoundHttpException;

class PointSaleController extends BackendController
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::class,
                'rules' => [
                    [
                        'allow' => true,
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return $this->getUserModule()
                                ->getAuthorizationChecker()
                                ->isGrantedAsProducer($this->getUserCurrent());
                        }
                    ],
                ],
            ],
        ];
    }

    /**
     * Liste les points de vente.
     */
    public function actionIndex()
    {
        $searchModel = new PointSaleSearch();
        $dataProvider = $searchModel->search(\Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Crée un point de vente.
     *
     * @return mixed
     */
    public function actionCreate()
    {
        $pointSaleModule = $this->getPointSaleModule();
        $distributionModule = $this-> getDistributionModule();

        $pointSale = $pointSaleModule->instanciatePointSale();

        if ($pointSale->load(\Yii::$app->request->post()) && $pointSale->save()) {
            $pointSaleModule->updatePointSalePointProduction($pointSale);
            $pointSaleModule->processRestrictedAccess($pointSale);
            $distributionModule->addPointSaleIncomingDistributions($pointSale);

            return $this->redirect(['index']);
        } else {
            return $this->render('create', array_merge($this->initForm(), [
                'model' => $pointSale,
            ]));
        }
    }

    /**
     * Modifie un point de vente.
     */
    public function actionUpdate(int $id)
    {
        $distributionModule = $this-> getDistributionModule();
        $pointSaleModule = $this->getPointSaleModule();
        $model = $this->findModel($id);

        foreach ($model->userPointSale as $userPointSale) {
            $model->users[] = $userPointSale->id_user;
            $model->users_comment[$userPointSale->id_user] = $userPointSale->comment;
        }

        if ($model->load(\Yii::$app->request->post()) && $model->save()) {

            $pointSaleModule->updatePointSalePointProduction($model);
            $pointSaleModule->processRestrictedAccess($model);
            $distributionModule->addPointSaleIncomingDistributions($model);

            $this->setFlash('success', 'Point de vente modifié.');

            return $this->redirect(['index']);
        } else {
            return $this->render('update', array_merge($this->initForm($id), [
                'model' => $model,
            ]));
        }
    }

    /**
     * Initialise le formulaire de création/modification.
     */
    public function initForm(int $id = 0)
    {
        $userModule = $this->getUserModule();

        $users = $userModule->queryUsersBy()
            ->leftJoin('user_point_sale', 'user_point_sale.id_user = user.id AND user_point_sale.id_point_sale = :id_point_sale', [':id_point_sale' => $id])
            ->orderBy('user_point_sale.id_point_sale DESC, lastname ASC, name ASC')
            ->all();

        return [
            'users' => $users
        ];
    }

    /**
     * Supprime un point de vente et redirige vers la liste des points de vente.
     */
    public function actionDelete(int $id, $confirm = false)
    {
        $orderModule = $this->getOrderModule();
        $distributionModule = $this-> getDistributionModule();

        $pointSale = $this->findModel($id);

        if ($confirm) {
            $pointSale->status = -1;
            $pointSale->save();

            // Suppression du lien entre les utilisateurs et le point de vente
            UserPointSale::deleteAll(['id_point_sale' => $id]);

            // Suppression du lien PointSaleDistribution pour toutes les distributions à venir
            $incomingDistributions = $distributionModule->findDistributionsIncoming(true);
            foreach ($incomingDistributions as $distribution) {
                PointSaleDistribution::deleteAll(['id_point_sale' => $id, 'id_distribution' => $distribution->id]);
            }

            // Suppression de toutes les commandes à venir de ce point de vente
            $ordersArray = Order::searchAll(
                [
                    'id_point_sale' => $id,
                ],
                [
                    'conditions' => 'date_delete IS NULL AND distribution.date > :today',
                    'params' => [':today' => date('Y-m-d')]
                ]
            );

            if ($ordersArray) {
                foreach ($ordersArray as $order) {
                    $orderModule->deleteOrder($order, true);
                }
            }

            $this->setFlash('success', 'Point de vente <strong>' . Html::encode($pointSale->name) . '</strong> supprimé.');
        } else {
            $this->setFlash('info', 'Souhaitez-vous vraiment supprimer le point de vente <strong>' . Html::encode($pointSale->name) . '</strong> ? '
                . Html::a('Oui', ['point-sale/delete', 'id' => $id, 'confirm' => 1], ['class' => 'btn btn-default']) . ' ' . Html::a('Non', ['point-sale/index'], ['class' => 'btn btn-default']));
        }

        return $this->redirect(['index']);
    }

    /**
     * Définit un point de vente par défaut.
     */
    public function actionDefault(int $id)
    {
        $pointSaleModule = $this->getPointSaleModule();

        $pointSale = $this->findModel($id);
        if ($pointSale) {
            PointSale::updateAll(['default' => 0], 'id_producer = :id_producer', [':id_producer' => GlobalParam::getCurrentProducerId()]);

            if (!$pointSale->default) {
                $pointSale->default = 1;
                $pointSaleModule->saveUpdate($pointSale);

                $this->setFlash('success', 'Point de vente <strong>' . Html::encode($pointSale->name) . '</strong> défini par défaut.');
            } else {
                $this->setFlash('success', 'Aucun point de vente défini par défaut');
            }
        }

        return $this->redirect(['index']);
    }

    /**
     * Recherche un point de vente en fonction de son ID.
     */
    protected function findModel(int $id)
    {
        $model = PointSale::find()
            ->with('userPointSale')
            ->where(['id' => $id])
            ->one();

        if(!$model || $model->id_producer != $this->getProducerCurrent()->id) {
            throw new NotFoundHttpException('Le point de vente demandé est introuvable.');
        }

        return $model;
    }
}
